// transfers.cpp by Matt Ownby

#include "transfers.h"

#include <mpolib/mpo_misc.h>

#include <wx/app.h>	// for wxYield

//////////////////////

// how often to refresh the stats
const unsigned int REFRESH_INTERVAL_MS = 1000;

//////////////////////

BEGIN_EVENT_TABLE(XferFrame, wxFrame)
EVT_MENU(ID_XferClose, XferFrame::OnClose)
END_EVENT_TABLE()

XferFrame::XferFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size),
	m_last_refresh_time(0), m_panel(NULL)
{
	CreateStatusBar(1);	// create a status bar with one field

	m_panel = new wxPanel(this);	// create our panel

	m_gsizer = new wxFlexGridSizer(5, 8, 5, 5);	// rows, cols, vgap, hgap

	AddTextFlex(NULL, "Uploaded");
	AddTextFlex(&m_statUploaded, " ");
	AddTextFlex(NULL, "Up k/s");
	AddTextFlex(&m_statULRate, " ");
	AddTextFlex(NULL, "Seeders");
	AddTextFlex(&m_statSeeders, " ");
	AddTextFlex(NULL, "Info Hash");
	AddTextFlex(&m_statInfoHash, " ");

	AddTextFlex(NULL, "Downloaded");
	AddTextFlex(&m_statDownloaded, " ");
	AddTextFlex(NULL, "Down k/s");
	AddTextFlex(&m_statDLRate, " ");
	AddTextFlex(NULL, "Leechers");
	AddTextFlex(&m_statLeechers, " ");
	AddTextFlex(NULL, "Good Piece %");
	AddTextFlex(&m_statGoodPiecePrcnt, " ");

	AddTextFlex(NULL, "Up/Down Ratio");
	AddTextFlex(&m_statUDRatio, " ");
	AddTextFlex(NULL, "Time Remaining");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, "Piece Size");
	AddTextFlex(&m_statPieceSize, " ");
	AddTextFlex(NULL, "Status");
	AddTextFlex(&m_statMsg, " ");

	AddTextFlex(NULL, "% Complete");
	AddTextFlex(&m_statPrcntComplete, " ");
	AddTextFlex(NULL, "Time Elapsed");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, "Piece Count");
	AddTextFlex(&m_statPieceCount, " ");
	AddTextFlex(NULL, "Next Tracker Update");
	AddTextFlex(&m_statNextUpdateS, " ");

	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, " ");
	AddTextFlex(NULL, "(update tracker)");
	AddTextFlex(NULL, " ");

	// START LIST CTRL HERE
	InitListCtrl();
	// END LIST CTRL

	// MAIN SIZERS
	m_vsizer = new wxBoxSizer(wxVERTICAL);
	m_vsizer->Add(m_gsizer, 0, wxLEFT | wxTOP, 10);
	m_vsizer->Add(m_listCtrl, 1, wxEXPAND | wxALL, 10);

	m_panel->SetSizer(m_vsizer);
//	m_vsizer->SetSizeHints(m_panel);	// not sure what this does
	m_panel->Fit();	// causes contents to fit together
	this->Fit();	// causes frame window to fit to smallest size around contents

}

XferFrame::~XferFrame()
{
	g_mainframe->XferFrameDying();	// tell mainframe that we're getting killed
}

// helper function
void XferFrame::AddTextFlex(wxStaticText **txt, const char *s)
{
	wxStaticText *text = NULL;
	
	text = new wxStaticText(m_panel, -1, _T(s));
	m_gsizer->Add(text, 0, wxEXPAND, 0);

	// if we want to hang onto this ptr
	if (txt != NULL) *txt = text;
}

void XferFrame::ChangeText(wxStaticText *txt, const string &s)
{
	txt->SetLabel(_T(s.c_str()));
	wxSize size = txt->GetSize();
	m_gsizer->SetItemMinSize(txt, size.x, size.y);	// resize this puppy
}

void XferFrame::InitListCtrl()
{
	m_listCtrl = new wxListCtrl(m_panel, ID_XferListCtrl,
		wxDefaultPosition, wxSize(800, 200),	// hack for now until I can figure out how to do it automagically
		wxLC_REPORT);

	const char *titles[] = {
		"IP", "Con", "Client", "%", "DL Rate", "DL Total", "UL Rate", "UL Total", 
			"Ratio", "Opt UL", "Opt Ratio", "RI", "RC", "LI", "LC", "Opt", "Snub", "% good",
			NULL
	};

	// initial values (to try to get columns auto-sized somewhat close)
	const char *vals[] = {
		"255.255.255.255", "Con", "Client Name Here", "100.0%", "9999.99 MiB/s ", "9999 MiB-",
			"9999.99 MiB/s", "9999 MiB-", 
			"0.000", "9999 MiB-", "Opt Ratio", "RI", "RC", "LI", "LC", "Opt", "Snub", "% good",
			NULL
	};

	m_listCtrl->Hide();	// make it faster and not so messy

	int i = 0;

	for (i = 0; titles[i] != NULL; i++)
	{
		m_listCtrl->InsertColumn(i, _T(titles[i]));
	}

	// fill in 1 row ...
	for (i = 0; i < 2; i++)
	{
		wxString buf;
		buf.Printf(_T("[empty]"));
		m_listCtrl->InsertItem(i, buf);
		
		for (int j = 0; vals[j] != NULL; j++)
		{
			m_listCtrl->SetItem(i, j, vals[j]);
		}
	}

	m_listCtrl->Show();

	for (i = 0; titles[i] != NULL; i++)
	{
		m_listCtrl->SetColumnWidth( i, wxLIST_AUTOSIZE );
	}

	m_listCtrl->Fit();

//	SendSizeEvent();	// don't know what this does, it was in an example
}

// helper function
inline const char *yn_helper(bool yn)
{
	if (yn) return "*";
	else return " ";
}

void XferFrame::refresh_stats(const torrent_stats &s, const string &info_hash)
{
	ChangeText(m_statUploaded, numstr::ToUnitStr(s.uploaded));
	ChangeText(m_statDownloaded, numstr::ToUnitStr(s.downloaded));
	ChangeText(m_statUDRatio, numstr::ToStr(s.ud_ratio, 0, 2, 2));
	ChangeText(m_statPrcntComplete, numstr::ToStr((100.0 * s.piece_count) / s.total_pieces, 0, 2, 2));

	ChangeText(m_statULRate, numstr::ToStr(s.ul_rate, 0, 2, 2));
	ChangeText(m_statDLRate, numstr::ToStr(s.dl_rate, 0, 2, 2));

	ChangeText(m_statSeeders, numstr::ToStr(s.seeders));
	ChangeText(m_statLeechers, numstr::ToStr(s.leechers));
	// NOTE : this only needs to be calculated once
	ChangeText(m_statPieceSize, numstr::ToUnitStr(s.piece_size));
	ChangeText(m_statPieceCount, numstr::ToStr(s.piece_count) + " of " +
		numstr::ToStr(s.total_pieces));

	ChangeText(m_statInfoHash, mpom::bin2hex(info_hash.data(), 20));

	// protect against divide by zero error
	unsigned int total_hashes = s.hash_successes + s.hash_fails;
	if (total_hashes > 0)
	{
		ChangeText(m_statGoodPiecePrcnt, numstr::ToStr((100.0 * s.hash_successes) / total_hashes, 0, 2, 2) +
			" (" + numstr::ToStr(s.hash_fails) + " hash fails)");
	}
	ChangeText(m_statMsg, s.status_msg);
	ChangeText(m_statNextUpdateS, numstr::ToStr(s.s_to_next_update));

//	m_listCtrl->Hide();

	size_t item_count = (size_t) m_listCtrl->GetItemCount();
	size_t p_idx = 0;

	// go through all peers and add them
	for (p_idx = 0; p_idx < (*s.peers).size(); p_idx++)
	{
		const torrent_peer *p = &((*s.peers)[p_idx]);
		wxString buf;
		buf.Printf(_T("[empty]"));

		// if we can't modify anymore pre-existing items, then add new items to the end
		if (p_idx >= item_count)
		{
			m_listCtrl->InsertItem(p_idx, buf);
		}

		// do IP
		m_listCtrl->SetItem(p_idx, COL_IP, p->ip.c_str());

		// do con
		if (p->incoming) buf = "R";
		else buf = "L";
		m_listCtrl->SetItem(p_idx, COL_CON, buf);

		// client
		m_listCtrl->SetItem(p_idx, COL_CLIENT, p->client_version.c_str());

		// percentage completed
		double prcnt_complete = (100.0 * p->remote_piece_count) / s.total_pieces;
		m_listCtrl->SetItem(p_idx, COL_PRCNT_FINISHED, numstr::ToStr(prcnt_complete, 0, 1, 2).c_str());

		// DL speed
		m_listCtrl->SetItem(p_idx, COL_DL_RATE, numstr::ToStr(p->last_dl_rate, 0, 1, 1).c_str());

		// UL speed
		m_listCtrl->SetItem(p_idx, COL_UL_RATE, numstr::ToStr(p->last_ul_rate, 0, 1, 1).c_str());

		// overall download
		m_listCtrl->SetItem(p_idx, COL_DL, numstr::ToUnitStr(p->downloaded).c_str());

		// overall upload
		m_listCtrl->SetItem(p_idx, COL_UL, numstr::ToUnitStr(p->uploaded).c_str());

		double ratio = 0.0, opt_ratio = 0.0;
		// avoid divide by zero error
		if (p->downloaded != 0)
		{
			ratio = ((double) p->uploaded) / p->downloaded;
			opt_ratio = (p->uploaded + p->opt_uploaded) / ((double) p->downloaded);
		}
		m_listCtrl->SetItem(p_idx, COL_UD_RATIO, numstr::ToStr(ratio, 0, 2, 2).c_str());

		// opt upload
		m_listCtrl->SetItem(p_idx, COL_OPT_UL, numstr::ToUnitStr(p->opt_uploaded).c_str());

		// opt upload ratio
		m_listCtrl->SetItem(p_idx, COL_OPT_UD_RATIO, numstr::ToStr(opt_ratio, 0, 2, 2).c_str());

		// RI
		m_listCtrl->SetItem(p_idx, COL_RI, yn_helper(p->remote_interested));

		// RC
		m_listCtrl->SetItem(p_idx, COL_RC, yn_helper(!p->remote_unchoked));

		// LI
		m_listCtrl->SetItem(p_idx, COL_LI, yn_helper(p->local_interested));

		// LC
		m_listCtrl->SetItem(p_idx, COL_LC, yn_helper(!p->local_unchoked));

		// optimistic unchoke
		m_listCtrl->SetItem(p_idx, COL_OPT, yn_helper(p->optimistic));

		// snubbed
		m_listCtrl->SetItem(p_idx, COL_SNUB, yn_helper(p->snubbed));
	}

	// if we originally had more items than columns to display, nuke them here ...
	while (p_idx < (size_t) m_listCtrl->GetItemCount())
	{
		m_listCtrl->DeleteItem(p_idx);
	}

//	m_listCtrl->Show();

	m_vsizer->Layout();	// necessary to update for dynamic changes
}

void XferFrame::refresh_group_stats(const torrent_group_stats &s)
{
	const the_vars *v = g_mainframe->get_readonly_vars();
	
	string msg = "UL: " + numstr::ToUnitStr(s.ul_bytes_per_sec) + "/s ";

	// if there is an upload cap, display it
	if (v->ul_bytes_per_s != 0)
	{
		msg += "[max " + numstr::ToUnitStr(v->ul_bytes_per_s) + "/s] ";
	}
		
	msg += "/ DL : " + numstr::ToUnitStr(s.dl_bytes_per_sec) + "/s ";
	
	// if there is a download cap, display it
	if (v->dl_bytes_per_s != 0)
	{
		msg += "[max " + numstr::ToUnitStr(v->dl_bytes_per_s) + "/s] ";
	}
	
	SetStatusText(msg.c_str(), 0);
}

void XferFrame::OnClose(wxCommandEvent &event)
{
}







XferControl::XferControl() :
	m_initialized(false),
	m_listen_port((unsigned int) -1),	// to ensure it gets initialized
	m_refresh_timer(0)
{
	// setup torrent to use current variables
	// NOTE ; as long as m_listen_port is different from bt_listen_port from the_vars, apply_cur_vars will call Init(),
	// so we just need to make sure m_listen_port is a unique value (we initialize it to -1 above)
	apply_cur_vars();	// setup torrent to use current variables
	Start(100);	// start calling Notify
}

XferControl::~XferControl()
{
	Stop();	// stop Notify from being called
}

// NOTE : it is safe to call Init multiple times without 'shutting down' because the it can re-initialize itself,
// and the various destructors (of m_gt for example) shut everything down properly
// WARNING : Init() MUST NOT call apply_cur_vars() because apply_cur_vars may call Init() and this would be a disastrous
//	circular loop
void XferControl::Init()
{
	mpo_io *rio;
	bool res = false;
	string info_hash = "";
	string err_msg = "(no err msg)";
	const the_vars *v = g_mainframe->get_readonly_vars();

	const char *TDIR = "./";	// current directory
	const char *TNAME = "./dlwad.tar.torrent";	// non-copyrighted pictures I took
	unlink("./dlwad.tar");	// to ensure that my testers have to re-download this file every time

//	const char *TNAME = "c:/milli.mp3.torrent";
//	const char *TDIR = "c:/temp";

//	const char *TNAME = "c:/nero.torrent";

//	const char *TNAME = "c:/mame.torrent";

//	const char *TNAME = "/tmp/milli.mp3.torrent";
//	const char *TDIR = "/tmp";
//	const char *TNAME = "/tmp/nero.torrent";

	// if we were already initialized, calling m_gt.initialize will close all open torrents and rebind to a new port,
	// therefore we should clear all our info hashes to keep our book keeping up to date.
	m_info_hashes.clear();

	m_initialized = m_gt.initialize(v->bt_listen_port);

	if (m_initialized)
	{
		m_listen_port = v->bt_listen_port;
		rio = mpo_open(TNAME, MPO_OPEN_READONLY);
		
		if (rio)
		{
			unsigned char *buf = new unsigned char[(size_t) rio->size];
			if (mpo_read(buf, (unsigned int) rio->size, NULL, rio))
			{
				res = m_gt.open((const char *) buf, (unsigned int) rio->size,
					TDIR, info_hash, err_msg);
					
				// if the torrent was opened successfully, add it to our queue
				if (res)
				{
					m_info_hashes.push_back(info_hash);
				}
					
			} // end if read succeeded
			delete [] buf;
			mpo_close(rio);
		}
	}
	else
	{
		m_listen_port = (unsigned int) -1;
		string s = "Could not listen on port " + numstr::ToStr(v->bt_listen_port) + ".\n"
			"This means that you will not be able to auto-download files.\n"
			"You can change the listen port in the Preferences.";
		g_mainframe->do_error(s.c_str());
	}
}

void XferControl::Notify()
{
	string err_msg = "no error";

	if (m_initialized)
	{
		unsigned int mid_timer = refresh_timer();

		// run a bunch of updates before sleeping to get this thing moving
		do
		{
			bool more_work_avail = false;
			if (!m_gt.update(more_work_avail, err_msg))
			{
				break;
			}

			// if there is nothing left for us to do for a while, then take a break ...
			if (!more_work_avail) break;

		} while (!time_elapsed(50, mid_timer));

		// this annoyingly causes dropdown menus to close, so.. it's not gonna work
//		wxSafeYield();

		g_mainapp->do_yield();

		// if the xfer frame is being displayed right now
		if (g_mainframe->m_xfer_frame != NULL)
		{
			// if it's time to refresh the stats for display purposes
			if (time_elapsed(REFRESH_INTERVAL_MS, m_refresh_timer))
			{
				// FIXME : this is just a hack to get this thing working .. it will need to be improved upon later
				if (m_gt.get_torrent_stats(m_cur_stats, m_info_hashes[0]))
				{
					g_mainframe->m_xfer_frame->refresh_stats(m_cur_stats,
						m_info_hashes[0]);
				}

				if (m_gt.get_group_stats(m_cur_group_stats))
				{
					g_mainframe->m_xfer_frame->refresh_group_stats(m_cur_group_stats);
				}

				m_refresh_timer = refresh_timer();
			}
		}
	}
}

const torrent_stats *XferControl::get_cur_stats()
{
	return &m_cur_stats;
}

// WARNING : this function can call Init() so Init() should _NOT_ call this function!
void XferControl::apply_cur_vars()
{
#ifdef DEBUG
	assert(g_mainframe != NULL);
#endif

	// must be read-write because if we find an invalid value, we need to change it
	the_vars *v = g_mainframe->get_readwrite_vars();

	// if our upload cap is too low
	if ((v->ul_bytes_per_s != 0) && (v->ul_bytes_per_s < 1024))
	{
		v->ul_bytes_per_s = 1024;
		g_mainframe->do_error("Upload max cannot be below 1.0 KiB/s");
	}
	
	m_gt.set_ul_limit(v->ul_bytes_per_s);
	
	// if our download cap is too low
	if ((v->dl_bytes_per_s != 0) && (v->dl_bytes_per_s < 1024))
	{
		v->dl_bytes_per_s = 1024;
		g_mainframe->do_error("Download max cannot be below 1.0 KiB/s");
	}

	m_gt.set_dl_limit(v->dl_bytes_per_s);
	
	// range check
	if ((unsigned int) v->leech_resistance > 2)
	{
		v->leech_resistance = 0;	// reset to auto
	}
	// TODO : set leech resistance
	
	// TODO : max peers

	// range check!
	if (v->bt_listen_port > 65535)
	{
		string s = "Requested port (" + numstr::ToStr(v->bt_listen_port) + ") is out of range.\n"
		"Resetting to default port range.";
		g_mainframe->do_error(s.c_str());
		v->bt_listen_port = 0;
	}
		
	// if the listen port has changed, then rebind and re-init
	if (m_listen_port != v->bt_listen_port)
	{
		Init();
	}
	
}
